@@ -1,7 +1,12 @@
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <gnome.h>
+#include "intl.h"
 
 #include "range2d.h"
 
@@ -29,6 +34,11 @@
 #define DEFAULT_RA  0.05
 #define DEFAULT_RB  0.05
 
+enum {
+  VALUE_CHANGED,
+  LAST_SIGNAL
+};
+
 static void gtk_range2d_class_init(GtkRange2dClass* klass);
 static void gtk_range2d_init(GtkRange2d* range2d);
 static void gtk_range2d_destroy(GtkObject* object);
@@ -46,8 +56,8 @@ static void configure_gnome_canvas_items(GtkRange2d* range2d, GnomeCanvas* canva
 static void entry_activate(GtkWidget* widget, gpointer arg);
 
 static GtkWindowClass* parent_class = NULL;
+static guint range2d_signals[LAST_SIGNAL] = { 0 };
 
-/***** public functions *****/
 
 GtkType gtk_range2d_get_type(void)
 {
@@ -83,11 +93,21 @@ void
 gtk_range2d_set_value(GtkRange2d* range2d,
 		      double a, double b, double ra, double rb)
 {
+  char buf[32];
   range2d->parameter[0] = a;
   range2d->parameter[1] = b;
   range2d->parameter[2] = ra;
   range2d->parameter[3] = rb;
 
+  sprintf(buf, "%f", range2d->parameter[0]);
+  gtk_entry_set_text(GTK_ENTRY(range2d->entry_a), buf);
+  sprintf(buf, "%f", range2d->parameter[1]);
+  gtk_entry_set_text(GTK_ENTRY(range2d->entry_b), buf);
+  sprintf(buf, "%f", range2d->parameter[2]);
+  gtk_entry_set_text(GTK_ENTRY(range2d->entry_ra), buf);
+  sprintf(buf, "%f", range2d->parameter[3]);
+  gtk_entry_set_text(GTK_ENTRY(range2d->entry_rb), buf);
+
   set_parameter (NULL, range2d);
 }
 
@@ -101,8 +121,6 @@ gtk_range2d_get_value(GtkRange2d* range2d,
   *rb = range2d->parameter[3];
 }
 
-/***** private functions *****/
-
 static void gtk_range2d_class_init(GtkRange2dClass* klass)
 {
   GtkObjectClass* object_class;
@@ -116,6 +134,18 @@ static void gtk_range2d_class_init(GtkRange2dClass* klass)
   object_class->destroy = gtk_range2d_destroy;
   object_class->finalize = gtk_range2d_finalize;
 
+  range2d_signals[VALUE_CHANGED] =
+    gtk_signal_new("value_changed",
+		   GTK_RUN_FIRST,
+		   object_class->type,
+		   GTK_SIGNAL_OFFSET(GtkRange2dClass, value_changed),
+		   gtk_marshal_NONE__NONE,
+		   GTK_TYPE_NONE, 0);
+
+  gtk_object_class_add_signals(object_class, range2d_signals, LAST_SIGNAL);
+
+  klass->value_changed = NULL;
+
   return;
 }
 
@@ -153,6 +183,8 @@ static void gtk_range2d_init(GtkRange2d* range2d)
    gtk_table_attach(GTK_TABLE(range2d->table1), range2d->canvas, 1, 2, 1, 2,
                      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 
+
+
    /* create a horizontal ruler */
    range2d->hrule = gtk_hruler_new();
    gtk_ruler_set_metric(GTK_RULER(range2d->hrule), GTK_PIXELS);
@@ -204,14 +236,18 @@ static void gtk_range2d_init(GtkRange2d* range2d)
   sprintf(buf, "%f", range2d->parameter[3]);
   gtk_entry_set_text(GTK_ENTRY(range2d->entry_rb), buf);
 
-  range2d->label_a = gtk_label_new(" = ");
-  range2d->label_b = gtk_label_new(" = ");
-  range2d->label_ra = gtk_label_new("r = ");
-  range2d->label_rb = gtk_label_new("r = ");
+  setlocale (LC_ALL, "");
+  bindtextdomain (PACKAGE, LOCALEDIR);
+  textdomain (PACKAGE);
 
-  range2d->button_save = gtk_button_new_with_label("save");
+  range2d->label_a = gtk_label_new(_("a = "));
+  range2d->label_b = gtk_label_new(_("b = "));
+  range2d->label_ra = gtk_label_new(_("ra = "));
+  range2d->label_rb = gtk_label_new(_("rb = "));
+
+  range2d->button_save = gtk_button_new_with_label(_("save"));
   gtk_widget_set_usize(GTK_WIDGET(range2d->button_save), 60, 20);
-  range2d->button_load = gtk_button_new_with_label("load");
+  range2d->button_load = gtk_button_new_with_label(_("load"));
   gtk_widget_set_usize(GTK_WIDGET(range2d->button_load), 60, 20);
 
   range2d->hbox = gtk_hbox_new(FALSE, 0);
@@ -429,9 +465,19 @@ static gint scope_move_by_drag(GnomeCanvasItem *item, GdkEvent *event, gpointer
 			       "x2", x2+dx, "y2", y2+dy, NULL);
 	gnome_canvas_item_set(range2d->ellipse, "x1", x1+dx, "y1", y1+dy,
 			       "x2", x2+dx, "y2", y2+dy, NULL);
+	gnome_canvas_item_set(range2d->pick00, 
+	       "x1", x1+dx, "y1", y1+dy,
+ 	       "x2", x1+dx + PICKER_SIZE, "y2", y1+dy + PICKER_SIZE, NULL);
+	gnome_canvas_item_set(range2d->pick01, 
+    	       "x1", x2+dx - PICKER_SIZE, "y1", y1+dy,
+	       "x2", x2+dx, "y2", y1+dy + PICKER_SIZE, NULL);
+	gnome_canvas_item_set(range2d->pick10, 
+	       "x1", x1+dx, "y1", y2+dy - PICKER_SIZE,
+	       "x2", x1+dx + PICKER_SIZE, "y2", y2+dy, NULL);
 	gnome_canvas_item_set(range2d->pick11, 
 	       "x1", x2+dx - PICKER_SIZE, "y1", y2+dy - PICKER_SIZE,
 	       "x2", x2+dx, "y2", y2+dy, NULL);
+
 	range2d->x0 = x;
 	range2d->y0 = y;
 
@@ -447,6 +493,7 @@ static gint scope_move_by_drag(GnomeCanvasItem *item, GdkEvent *event, gpointer
 	gtk_entry_set_text(GTK_ENTRY(range2d->entry_ra), buf);
 	sprintf(buf, "%f", range2d->parameter[3]);
 	gtk_entry_set_text(GTK_ENTRY(range2d->entry_rb), buf);
+	gtk_range2d_value_changed(range2d);
       }
       return TRUE;
 
@@ -461,6 +508,7 @@ static gint scope_move_by_drag(GnomeCanvasItem *item, GdkEvent *event, gpointer
 }
 
 /* resize scope */
+
 static gint scope_resize_by_drag(GnomeCanvasItem *item, GdkEvent *event, gpointer arg)
 {
   double x1, y1, x2, y2;
@@ -492,14 +540,42 @@ static gint scope_resize_by_drag(GnomeCanvasItem *item, GdkEvent *event, gpointe
 	if ((x2+x-range2d->x0 - x1) < MINIMUM_SCOPE_SIZE) return FALSE;
 	if ((y2+y-range2d->y0 - y1) < MINIMUM_SCOPE_SIZE) return FALSE;
 
-	gnome_canvas_item_set(range2d->rect, "x2", x2 + x - range2d->x0,
-			             "y2", y2 + y - range2d->y0, NULL);
-	gnome_canvas_item_set(range2d->ellipse, "x2", x2 + x - range2d->x0,
-			             "y2", y2 + y - range2d->y0, NULL);
+	if(item == range2d->pick00) {
+	  gnome_canvas_item_set(range2d->rect, "x1", x1 + x - range2d->x0,
+				"y1", y1 + y - range2d->y0, NULL);
+	  gnome_canvas_item_set(range2d->ellipse, "x1", x1 + x - range2d->x0,
+				"y1", y1 + y - range2d->y0, NULL);
+	} else if (item == range2d->pick01) {
+	  gnome_canvas_item_set(range2d->rect, "x2", x2 + x - range2d->x0,
+				"y1", y1 + y - range2d->y0, NULL);
+	  gnome_canvas_item_set(range2d->ellipse, "x2", x2 + x - range2d->x0,
+				"y1", y1 + y - range2d->y0, NULL);
+	} else if (item == range2d->pick10) {
+	  gnome_canvas_item_set(range2d->rect, "x1", x1 + x - range2d->x0,
+				"y2", y2 + y - range2d->y0, NULL);
+	  gnome_canvas_item_set(range2d->ellipse, "x1", x1 + x - range2d->x0,
+				"y2", y2 + y - range2d->y0, NULL);
+	} else if (item == range2d->pick11) {
+	  gnome_canvas_item_set(range2d->rect, "x2", x2 + x - range2d->x0,
+				"y2", y2 + y - range2d->y0, NULL);
+	  gnome_canvas_item_set(range2d->ellipse, "x2", x2 + x - range2d->x0,
+				"y2", y2 + y - range2d->y0, NULL);
+	}
+	  
 	gnome_canvas_item_get_bounds(range2d->rect, &x1, &y1, &x2, &y2);
+	gnome_canvas_item_set(range2d->pick00, 
+	       "x1", x1, "y1", y1,
+ 	       "x2", x1 + PICKER_SIZE, "y2", y1 + PICKER_SIZE, NULL);
+	gnome_canvas_item_set(range2d->pick01, 
+    	       "x1", x2 - PICKER_SIZE, "y1", y1,
+	       "x2", x2, "y2", y1 + PICKER_SIZE, NULL);
+	gnome_canvas_item_set(range2d->pick10, 
+	       "x1", x1, "y1", y2 - PICKER_SIZE,
+	       "x2", x1 + PICKER_SIZE, "y2", y2, NULL);
 	gnome_canvas_item_set(range2d->pick11, 
 	       "x1", x2 - PICKER_SIZE, "y1", y2 - PICKER_SIZE,
 	       "x2", x2, "y2", y2, NULL);
+
 	range2d->x0 = x;
 	range2d->y0 = y;
 
@@ -515,11 +591,21 @@ static gint scope_resize_by_drag(GnomeCanvasItem *item, GdkEvent *event, gpointe
 	gtk_entry_set_text(GTK_ENTRY(range2d->entry_ra), buf);
 	sprintf(buf, "%f", range2d->parameter[3]);
 	gtk_entry_set_text(GTK_ENTRY(range2d->entry_rb), buf);
+	gtk_range2d_value_changed(range2d);
       }
       return TRUE;
 
     case GDK_BUTTON_RELEASE:
       range2d->resize_dragged = FALSE;
+      gnome_canvas_item_set(range2d->pick00, 
+	     "x1", x1, "y1", y1,
+ 	     "x2", x1 + PICKER_SIZE, "y2", y1 + PICKER_SIZE, NULL);
+      gnome_canvas_item_set(range2d->pick01, 
+    	     "x1", x2 - PICKER_SIZE, "y1", y1,
+	     "x2", x2, "y2", y1 + PICKER_SIZE, NULL);
+      gnome_canvas_item_set(range2d->pick10, 
+	     "x1", x1, "y1", y2 - PICKER_SIZE,
+	     "x2", x1 + PICKER_SIZE, "y2", y2, NULL);
       gnome_canvas_item_set(range2d->pick11, 
 	     "x1", x2 - PICKER_SIZE, "y1", y2 - PICKER_SIZE,
 	     "x2", x2, "y2", y2, NULL);
@@ -547,6 +633,15 @@ static gint set_parameter(GtkWidget* widget, gpointer arg)
   gnome_canvas_item_set(range2d->ellipse, 
 			 "x1", x1, "y1", y1,
 			 "x2", x2, "y2", y2, NULL);
+  gnome_canvas_item_set(range2d->pick00, 
+			"x1", x1, "y1", y1,
+			"x2", x1 + PICKER_SIZE, "y2", y1 + PICKER_SIZE, NULL);
+  gnome_canvas_item_set(range2d->pick01, 
+			"x1", x2 - PICKER_SIZE, "y1", y1,
+			"x2", x2, "y2", y1 + PICKER_SIZE, NULL);
+  gnome_canvas_item_set(range2d->pick10, 
+			"x1", x1, "y1", y2 - PICKER_SIZE,
+			"x2", x1 + PICKER_SIZE, "y2", y2, NULL);
   gnome_canvas_item_set(range2d->pick11, 
 			 "x1", x2 - PICKER_SIZE, "y1", y2 - PICKER_SIZE,
 			 "x2", x2, "y2", y2, NULL);
@@ -558,8 +653,21 @@ static
 void
 configure_gnome_canvas_items(GtkRange2d* range2d, GnomeCanvas* canvas)
 {
-  range2d->root = gnome_canvas_root(GNOME_CANVAS(canvas));
+  GdkImlibImage *img;
+  img = gdk_imlib_load_image("triangle.png");
 
+  range2d->root = gnome_canvas_root(GNOME_CANVAS(canvas));
+  /*
+  range2d->image = gnome_canvas_item_new (range2d->root,
+				 gnome_canvas_image_get_type(),
+				 "image", img,
+				 "x", 10.0,
+				 "y", 10.0,
+				 "width", (double) img->rgb_width,
+				 "height", (double) img->rgb_height,
+				 "anchor", GTK_ANCHOR_CENTER,
+				 NULL);
+  */
   range2d->rect = gnome_canvas_item_new(range2d->root, 
 				gnome_canvas_rect_get_type(),
 				"x1", SCOPE_DEFAULT_X1, 
@@ -579,6 +687,33 @@ configure_gnome_canvas_items(GtkRange2d* range2d, GnomeCanvas* canvas)
 				"outline_color", DEFAULT_BORDER_COLOR, 
 				NULL);
 
+  range2d->pick00 = gnome_canvas_item_new(range2d->root, 
+				gnome_canvas_rect_get_type(),
+				"x1", SCOPE_DEFAULT_X1,
+				"y1", SCOPE_DEFAULT_Y1,
+				"x2", SCOPE_DEFAULT_X1 + PICKER_SIZE,
+			        "y2", SCOPE_DEFAULT_Y1 + PICKER_SIZE,
+				"fill_color", DEFAULT_BORDER_COLOR,
+				 NULL);
+
+  range2d->pick01 = gnome_canvas_item_new(range2d->root, 
+				gnome_canvas_rect_get_type(),
+				"x1", SCOPE_DEFAULT_X2 - PICKER_SIZE,
+				"y1", SCOPE_DEFAULT_Y1,
+				"x2", SCOPE_DEFAULT_X2,
+			        "y2", SCOPE_DEFAULT_Y1 + PICKER_SIZE,
+				"fill_color", DEFAULT_BORDER_COLOR,
+				 NULL);
+
+  range2d->pick10 = gnome_canvas_item_new(range2d->root, 
+				gnome_canvas_rect_get_type(),
+				"x1", SCOPE_DEFAULT_X1,
+			        "y1", SCOPE_DEFAULT_Y2 - PICKER_SIZE,
+				"x2", SCOPE_DEFAULT_X1 + PICKER_SIZE,
+			        "y2", SCOPE_DEFAULT_Y2,
+				"fill_color", DEFAULT_BORDER_COLOR,
+				 NULL);
+
   range2d->pick11 = gnome_canvas_item_new(range2d->root, 
 				gnome_canvas_rect_get_type(),
 				"x1", SCOPE_DEFAULT_X2 - PICKER_SIZE,
@@ -586,12 +721,43 @@ configure_gnome_canvas_items(GtkRange2d* range2d, GnomeCanvas* canvas)
 				"x2", SCOPE_DEFAULT_X2, "y2", SCOPE_DEFAULT_Y2,
 				"fill_color", DEFAULT_BORDER_COLOR,
 				 NULL);
-
-  gtk_signal_connect(GTK_OBJECT(range2d->root), "event",
-		      GTK_SIGNAL_FUNC(scope_move_by_drag), range2d);
-
+  /*
+  gnome_canvas_item_grab(range2d->ellipse,
+			 GDK_MOTION_NOTIFY | GDK_BUTTON_PRESS
+			 | GDK_BUTTON_RELEASE,
+			 NULL,
+			 GDK_CURRENT_TIME);
+  gnome_canvas_item_grab(range2d->pick00,
+			 GDK_MOTION_NOTIFY | GDK_BUTTON_PRESS
+			 | GDK_BUTTON_RELEASE,
+			 NULL,
+			 GDK_CURRENT_TIME);
+  gnome_canvas_item_grab(range2d->pick01,
+			 GDK_MOTION_NOTIFY | GDK_BUTTON_PRESS
+			 | GDK_BUTTON_RELEASE,
+			 NULL,
+			 GDK_CURRENT_TIME);
+  gnome_canvas_item_grab(range2d->pick10,
+			 GDK_MOTION_NOTIFY | GDK_BUTTON_PRESS
+			 | GDK_BUTTON_RELEASE,
+			 NULL,
+			 GDK_CURRENT_TIME);
+  gnome_canvas_item_grab(range2d->pick11,
+			 GDK_MOTION_NOTIFY | GDK_BUTTON_PRESS
+			 | GDK_BUTTON_RELEASE,
+			 NULL,
+			 GDK_CURRENT_TIME);
+*/
+  gtk_signal_connect(GTK_OBJECT(range2d->ellipse), "event",
+		     GTK_SIGNAL_FUNC(scope_move_by_drag), range2d);
+  gtk_signal_connect(GTK_OBJECT(range2d->pick00), "event",
+		     GTK_SIGNAL_FUNC(scope_resize_by_drag), range2d);
+  gtk_signal_connect(GTK_OBJECT(range2d->pick01), "event",
+		     GTK_SIGNAL_FUNC(scope_resize_by_drag), range2d);
+  gtk_signal_connect(GTK_OBJECT(range2d->pick10), "event",
+		     GTK_SIGNAL_FUNC(scope_resize_by_drag), range2d);
   gtk_signal_connect(GTK_OBJECT(range2d->pick11), "event",
-		      GTK_SIGNAL_FUNC(scope_resize_by_drag), range2d);
+		     GTK_SIGNAL_FUNC(scope_resize_by_drag), range2d);
 }
 
 /* entry_activate event handler */
@@ -620,4 +786,13 @@ static void entry_activate(GtkWidget* widget, gpointer arg)
       sprintf(buf, "%f", range2d->parameter[3]);
       gtk_entry_set_text(GTK_ENTRY(range2d->entry_rb), buf);
   }
+  gtk_range2d_value_changed(range2d);
+}
+
+void gtk_range2d_value_changed(GtkRange2d* range2d)
+{
+  g_return_if_fail(range2d != NULL);
+  g_return_if_fail(GTK_IS_RANGE2D(range2d));
+
+  gtk_signal_emit(GTK_OBJECT(range2d), range2d_signals[VALUE_CHANGED]);
 }
